package cd.casic.ci.process.engine.manager.impl;

import cd.casic.ci.process.engine.enums.ContextStateEnum;
import cd.casic.ci.process.engine.manager.LoggerManager;
import cd.casic.ci.process.engine.manager.RunContextManager;
import cd.casic.ci.process.engine.runContext.BaseRunContext;
import cd.casic.ci.process.engine.runContext.PipelineRunContext;
import cd.casic.ci.process.engine.runContext.SecondStageRunContext;
import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.process.dal.pipeline.PipTaskDao;
import cd.casic.ci.process.process.dataObject.base.PipBaseElement;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Component
public class DefaultRunContextManager implements RunContextManager {
    private final Map<String,PipelineRunContext> contextMap= new ConcurrentHashMap();
    @Resource
    private LoggerManager loggerManager;
    @Resource
    private PipTaskDao taskDao;
    @Override
    public Boolean stopPipeline(String pipelineId) {
        return null;
    }

    @Override
    public Boolean notifyPipeline(String pipelineId) {
        return null;
    }

    @Override
    public Boolean suspendPipeline(String pipelineId) {
        return null;
    }

    @Override
    public Boolean notifyStage(String pipelineId, String stageId) {
        return null;
    }

    @Override
    public Boolean suspendStage(String pipelineId, String stageId) {
        PipelineRunContext pipelineRunContext = contextMap.get(pipelineId);
        if (pipelineRunContext !=null) {

        }
        return null;
    }

    @Override
    public void contextRegister(BaseRunContext context) {
        PipBaseElement contextDef = context.getContextDef();
        String id = contextDef.getId();
        BaseRunContext parentContext = context.getParentContext();
        if (context instanceof PipelineRunContext pipelineRunContext) {
            if (contextMap.containsKey(id)) {
                PipelineRunContext oldPipeline = contextMap.get(id);
                oldPipeline.changeContextStateAndChild(ContextStateEnum.BAD_ENDING);
                List<PipTask> taskList = taskDao.selectList("pipelineId", id);
                List<String> taskIdList = taskList.stream().map(PipTask::getId).toList();
                // 清空上一次的日志
                loggerManager.flushMemory(taskIdList);
            }
            contextMap.put(id,pipelineRunContext);
        } else {
            if (parentContext==null) {
                throw new ServiceException(GlobalErrorCodeConstants.PIPELINE_ERROR.getCode(),"注册context失败");
            }
            PipBaseElement parentContextDef = parentContext.getContextDef();
            BaseRunContext parentRunContext = null;
            for (Map.Entry<String, PipelineRunContext> entry : contextMap.entrySet()) {
                PipelineRunContext value = entry.getValue();
                BaseRunContext runContext = value.getRunContext(parentContextDef.getId());
                if (runContext!=null) {
                    parentRunContext = runContext;
                    break;
                }
            }
            if (parentRunContext==null) {
                throw new ServiceException(GlobalErrorCodeConstants.PIPELINE_ERROR.getCode(),"未找到父级context");
            }
            parentRunContext.putChildRunContext(id,context);
        }
    }

    @Override
    public BaseRunContext getContext(String key) {
        for (Map.Entry<String, PipelineRunContext> entry : contextMap.entrySet()) {
            PipelineRunContext value = entry.getValue();
            BaseRunContext runContext = value.getRunContext(key);
            if (runContext!=null) {
                return runContext;
            }
        }
        return null;
    }
    public void changePipelineState(String pipelineId,ContextStateEnum stateEnum){
        PipelineRunContext pipelineRunContext = contextMap.get(pipelineId);
        if (pipelineRunContext==null) {
            return;
        }
        pipelineRunContext.changeContextStateAndChild(stateEnum);
    }

}
