package cd.casic.ci.process.engine.runContext;

import cd.casic.ci.process.engine.enums.ContextStateEnum;
import cd.casic.ci.process.process.dataObject.base.PipBaseElement;
import cd.casic.ci.process.process.dataObject.pipeline.PipPipeline;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;

import java.time.LocalDateTime;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class PipelineRunContext extends BaseRunContext{

    public PipelineRunContext(PipPipeline pipeline,Integer childCount) {
        this(pipeline,childCount,null,new ConcurrentHashMap<>(),new ConcurrentHashMap<>());
    }

    public PipelineRunContext(PipPipeline pipeline,Integer childCount,BaseRunContext parentContext, Map<String, Object> globalVariables, Map<String, Object> localVariables) {
        this(pipeline,childCount,parentContext,LocalDateTime.now(),pipeline.getResourceId(),pipeline.getTargetVersionId(),pipeline.getTargetType(),globalVariables,localVariables,new ConcurrentHashMap<>());
    }

    private PipelineRunContext(PipBaseElement contextDef,Integer childCount, BaseRunContext parentContext, LocalDateTime startTime, String resourceId, String targetVersionId, String targetType, Map<String, Object> globalVariables, Map<String, Object> localVariables, Map<String, BaseRunContext> childContext) {
        super(  contextDef
                ,childCount
                ,parentContext
                ,startTime
                , resourceId
                , targetVersionId
                , targetType
                , globalVariables
                , localVariables
                , childContext);
    }

    /**
     *  pipeline 底下有多个阶段，多个阶段包含多个task 不保存第二级context
     * */
    @Override
    public BaseRunContext getRunContext(String id) {
        if (this.getContextDef().getId().equals(id)) {
            return this;
        }
        Map<String, BaseRunContext> childContext = getChildContext();
        for (Map.Entry<String, BaseRunContext> entry : childContext.entrySet()) {
            BaseRunContext childRunContext = entry.getValue().getRunContext(id);
            if (childRunContext!=null) {
                return childRunContext;
            }
        }
        return null;
    }

    @Override
    public void putChildRunContext(String key, BaseRunContext context) {
        Map<String, BaseRunContext> childContext = getChildContext();
        if (context instanceof SecondStageRunContext) {
            childContext.put(key,context);
        } else {
            throw new ServiceException(GlobalErrorCodeConstants.PIPELINE_ERROR.getCode(),"不支持类型");
        }
    }
}
