package cd.casic.ci.process.process.service.resource.impl;

import cd.casic.ci.process.dal.req.resource.ResourceQueryReq;
import cd.casic.ci.process.dal.req.resource.ResourceReq;
import cd.casic.ci.process.dal.resp.resource.ResourceFindResp;
import cd.casic.ci.process.process.converter.ResourceConverter;
import cd.casic.ci.process.process.dal.pipeline.PipResourceCloudDao;
import cd.casic.ci.process.process.dal.pipeline.PipResourceDockerEndpointDao;
import cd.casic.ci.process.process.dal.pipeline.PipResourceK8SDao;
import cd.casic.ci.process.process.dal.pipeline.PipResourceMachineDao;
import cd.casic.ci.process.process.dal.pipeline.PipResourceManagerDao;
import cd.casic.ci.process.process.dataObject.resource.PipResourceCloud;
import cd.casic.ci.process.process.dataObject.resource.PipResourceK8S;
import cd.casic.ci.process.process.dataObject.resource.PipResourceMachine;
import cd.casic.ci.process.process.dataObject.resource.PipResourceManager;
import cd.casic.ci.process.process.dataObject.resource.PipelineDockerEndpoint;
import cd.casic.ci.process.process.service.resource.ResourceManagerService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.security.dal.user.AdminUserDO;
import cd.casic.framework.tenant.core.service.AdminUserServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName ResourceServiceImpl
 * @Date: 2025/5/13 10:31
 * @Description:
 */
@Service
@Slf4j
public class ResourceManagerServiceImpl extends ServiceImpl<PipResourceManagerDao, PipResourceManager> implements ResourceManagerService {
    @Resource
    private PipResourceManagerDao resourceManagerDao;

    @Resource
    private PipResourceMachineDao machineDao;

    @Autowired
    private PipResourceDockerEndpointDao dockerEndpointDao;

    @Resource
    private PipResourceK8SDao k8SDao;

    @Resource
    private PipResourceCloudDao cloudDao;

    @Resource
    private AdminUserServiceImpl adminUserService;
    @Autowired
    private PipResourceDockerEndpointDao pipResourceDockerEndpointDao;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createResource(ResourceReq resourceReq) {

        PipResourceManager pipResourceManager = ResourceConverter.INSTANCE.toBean(resourceReq);

        //如果资源类型不属于弹性云，则新增，如果属于弹性云，则走申请流程
        if (!ObjectUtils.isEmpty(resourceReq.getResourceMachine())){
            PipResourceMachine resourceMachine = resourceReq.getResourceMachine();
            machineDao.insert(resourceMachine);

            pipResourceManager.setMachineId(resourceMachine.getId());
        }
        if (!ObjectUtils.isEmpty(resourceReq.getPipResourceK8S())) {
            PipResourceK8S pipResourceK8S = resourceReq.getPipResourceK8S();
            k8SDao.insert(pipResourceK8S);

            pipResourceManager.setK8sId(pipResourceK8S.getId());
        }
        if (!ObjectUtils.isEmpty(resourceReq.getDockerEndpoint())) {
            PipelineDockerEndpoint dockerEndpoint = resourceReq.getDockerEndpoint();
            dockerEndpointDao.insert(dockerEndpoint);

            pipResourceManager.setDockerId(dockerEndpoint.getId());
        }

        if (!ObjectUtils.isEmpty(resourceReq.getPipResourceCloud())) {
            PipResourceCloud pipResourceCloud = resourceReq.getPipResourceCloud();
            cloudDao.insert(pipResourceCloud);

            pipResourceManager.setCloudId(pipResourceCloud.getId());
        }

        resourceManagerDao.insert(pipResourceManager);
        return pipResourceManager.getId();
    }

    @Override
    public void deleteResource(String id) {
        PipResourceManager pipResourceManager = resourceManagerDao.selectById(id);
        if (!StringUtils.isEmpty(pipResourceManager.getMachineId())){
            machineDao.deleteById(pipResourceManager.getMachineId());
        }
        if (!StringUtils.isEmpty(pipResourceManager.getK8sId())){
            k8SDao.deleteById(pipResourceManager.getK8sId());
        }
        if (!StringUtils.isEmpty(pipResourceManager.getDockerId())){
            dockerEndpointDao.deleteById(pipResourceManager.getDockerId());
        }
        if (!StringUtils.isEmpty(pipResourceManager.getCloudId())){
            cloudDao.deleteById(pipResourceManager.getCloudId());
        }
        resourceManagerDao.deleteById(id);
    }

    @Override
    public void updateResource(ResourceReq resourceReq) {
        PipResourceManager temp = resourceManagerDao.selectById(resourceReq.getId());
        temp.setResourceName(resourceReq.getResourceName());

        //先删除旧的资源信息
        if (!StringUtils.isEmpty(temp.getMachineId())){
            machineDao.deleteById(temp.getMachineId());
        }


        if (!StringUtils.isEmpty(temp.getK8sId())){
            k8SDao.deleteById(temp.getK8sId());

        }

        if (!StringUtils.isEmpty(temp.getDockerId())){
            dockerEndpointDao.deleteById(temp.getDockerId());

        }

        if (!StringUtils.isEmpty(temp.getCloudId())){
            cloudDao.deleteById(temp.getCloudId());

        }


        //根据新资源信息新增
        if (!ObjectUtils.isEmpty(resourceReq.getResourceMachine())){
            PipResourceMachine resourceMachine = resourceReq.getResourceMachine();
            machineDao.insert(resourceMachine);

            temp.setMachineId(resourceMachine.getId());
        }
        if (!ObjectUtils.isEmpty(resourceReq.getPipResourceK8S())) {
            PipResourceK8S pipResourceK8S = resourceReq.getPipResourceK8S();
            k8SDao.insert(pipResourceK8S);

            temp.setK8sId(pipResourceK8S.getId());
        }
        if (!ObjectUtils.isEmpty(resourceReq.getDockerEndpoint())) {
            PipelineDockerEndpoint dockerEndpoint = resourceReq.getDockerEndpoint();
            dockerEndpointDao.insert(dockerEndpoint);

            temp.setDockerId(dockerEndpoint.getId());
        }

        if (!ObjectUtils.isEmpty(resourceReq.getPipResourceCloud())) {
            PipResourceCloud pipResourceCloud = resourceReq.getPipResourceCloud();
            cloudDao.insert(pipResourceCloud);

            temp.setCloudId(pipResourceCloud.getId());
        }
        resourceManagerDao.updateById(temp);
    }

    @Override
    public ResourceFindResp findResourceById(String id) {
        PipResourceManager pipResourceManager = resourceManagerDao.selectById(id);
        if (ObjectUtils.isEmpty(pipResourceManager)){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"资源不存在");
        }
        ResourceFindResp resp = ResourceConverter.INSTANCE.toResp(pipResourceManager);

        setResource(resp);
        setUserName(resp);
        return resp;
    }



    @Override
    public List<ResourceFindResp> findResourceList(ResourceQueryReq query) {
        QueryWrapper<PipResourceManager> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(query.getId())){
            wrapper.eq("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getIdList())){
            wrapper.in("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getResourceName())){
            wrapper.like("resource_name",query.getResourceName());
        }
        if (!ObjectUtils.isEmpty(query.getMachineId())){
            wrapper.eq("machine_id",query.getMachineId());
        }
        if (!ObjectUtils.isEmpty(query.getK8sId())){
            wrapper.eq("k8s_id",query.getK8sId());
        }
        if (!ObjectUtils.isEmpty(query.getDockerId())){
            wrapper.eq("docker_id",query.getDockerId());
        }
        if (!ObjectUtils.isEmpty(query.getCloudId())){
            wrapper.eq("cloud_id",query.getCloudId());
        }
        if (!ObjectUtils.isEmpty(query.getCreator())){
            wrapper.eq("creator",query.getCreator());
        }
        List<PipResourceManager> pipResourceManagerList = resourceManagerDao.selectList(wrapper);

        if (ObjectUtils.isEmpty(pipResourceManagerList)){
            return new ArrayList<>(0);
        }

        List<ResourceFindResp> respList = ResourceConverter.INSTANCE.toRespList(pipResourceManagerList);

        //对资源进行赋值
        respList.forEach(this::setResource);
        respList.forEach(this::setUserName);

        return respList;
    }

    @Override
    public PageResult<ResourceFindResp> findResourcePage(ResourceQueryReq query) {
        Page<ResourceFindResp> respPage = new Page<>();

        QueryWrapper<PipResourceManager> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(query.getId())){
            wrapper.eq("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getIdList())){
            wrapper.in("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getResourceName())){
            wrapper.like("resource_name",query.getResourceName());
        }
        if (!ObjectUtils.isEmpty(query.getMachineId())){
            wrapper.eq("machine_id",query.getMachineId());
        }
        if (!ObjectUtils.isEmpty(query.getK8sId())){
            wrapper.eq("k8s_id",query.getK8sId());
        }
        if (!ObjectUtils.isEmpty(query.getDockerId())){
            wrapper.eq("docker_id",query.getDockerId());
        }
        if (!ObjectUtils.isEmpty(query.getCloudId())){
            wrapper.eq("cloud_id",query.getCloudId());
        }
        if (!ObjectUtils.isEmpty(query.getCreator())){
            wrapper.eq("creator",query.getCreator());
        }
        Page<PipResourceManager> pipResourceTempPage = resourceManagerDao.selectPage(new Page<>(query.getPageNo(), query.getPageSize()), wrapper);

        if (ObjectUtils.isEmpty(pipResourceTempPage)){
            return new PageResult<>();
        }

        List<ResourceFindResp> respList = ResourceConverter.INSTANCE.toRespList(pipResourceTempPage.getRecords());

        //对资源进行赋值
        respList.forEach(this::setResource);
        respList.forEach(this::setUserName);

        respPage.setRecords(respList);
        respPage.setTotal(pipResourceTempPage.getTotal());
        respPage.setCurrent(pipResourceTempPage.getCurrent());
        respPage.setSize(pipResourceTempPage.getSize());
        PageResult<ResourceFindResp> pageResult = new PageResult<>(respList,pipResourceTempPage.getTotal());
        return pageResult;
    }

    private void setResource(ResourceFindResp resourceFindResp) {
        if (!StringUtils.isEmpty(resourceFindResp.getMachineId())){
            PipResourceMachine resourceMachine = machineDao.selectById(resourceFindResp.getMachineId());
            resourceFindResp.setResourceMachine(resourceMachine);
        }
        if (!StringUtils.isEmpty(resourceFindResp.getK8sId())) {
            PipResourceK8S pipResourceK8S = k8SDao.selectById(resourceFindResp.getK8sId());
            resourceFindResp.setPipResourceK8S(pipResourceK8S);
        }
        if (!StringUtils.isEmpty(resourceFindResp.getDockerId())) {
            PipelineDockerEndpoint dockerEndpoint = dockerEndpointDao.selectById(resourceFindResp.getDockerId());
            resourceFindResp.setDockerEndpoint(dockerEndpoint);
        }
        if (!StringUtils.isEmpty(resourceFindResp.getCloudId())) {
            PipResourceCloud resourceCloud = cloudDao.selectById(resourceFindResp.getCloudId());
            resourceFindResp.setPipResourceCloud(resourceCloud);
        }
    }


    private void setUserName(ResourceFindResp resp) {
        if (!StringUtils.isEmpty(resp.getCreator())){
            AdminUserDO user = adminUserService.getUser(Long.valueOf(resp.getCreator()));
            if (!ObjectUtils.isEmpty(user)){
                resp.setCreatorName(user.getUsername());
            }
        }

        if (!StringUtils.isEmpty(resp.getUpdater())){
            AdminUserDO user = adminUserService.getUser(Long.valueOf(resp.getUpdater()));
            if (!ObjectUtils.isEmpty(user)){
                resp.setUpdaterName(user.getUsername());
            }
        }
    }
}
